#include <avr/io.h>
#include <avr/interrupt.h>
#include "integer.h"
#include "irda_io.h"
#include "crc8.h"




#define SIGN 0xA3
#define SNDSIZE 50

typedef struct _IRDA_RCVRC {
	BYTE sign;		
	BYTE key;
	BYTE rs;
	BYTE ps;
	BYTE tp;
	BYTE crc;
	DWORD size;
	BYTE name[40];		
} IRDA_RCVRC;

static BYTE ns=0;
static BYTE nr=0;
static BYTE irda_rcvfl=0;

static IRDA_RCVRC irda_rcvrec;
static BYTE *irda_rcvbf	=(BYTE *)(&irda_rcvrec);
static BYTE irda_sndbf[6];

void irda_initialize()
{

	UINT bf_addr;

	irda_rcvfl =IRDA_STATUS_DISABLE;

	/*  IrDa    */
	USARTC0.CTRLC     =  USART_CMODE_ASYNCHRONOUS_gc | USART_SBMODE_bm | USART_CHSIZE_8BIT_gc;
	USARTC0.BAUDCTRLB = 0x02;
	USARTC0.BAUDCTRLA = 0x2B;
	USARTC0.CTRLB	  = USART_RXEN_bm | USART_TXEN_bm; 
	
	cli();
	USARTC0.CTRLA	= USART_RXCINTLVL_LO_gc | USART_TXCINTLVL_LO_gc;
	sei(); 
}

/*     */
BYTE irda_status()
{
	return irda_rcvfl;
}


void irda_send(BYTE cm,BYTE pos)
{
	irda_sndbf[0]= SIGN;
	irda_sndbf[1]= cm;
	irda_sndbf[2]= pos;
	irda_sndbf[3]= Crc8(irda_sndbf,3);

	for(nr=0; nr < sizeof(IRDA_RCVRC); nr++) irda_rcvbf[nr]=0;
	nr=0;

	ns=0;
	PORTC_OUTCLR=0b00000010;
	TCC0.CTRLA 		= TC_CLKSEL_DIV8_gc;
	TCC0.CTRLB 		= TC0_CCBEN_bm | TC_WGMODE_FRQ_gc;


	irda_rcvfl = IRDA_STATUS_TRANSMITTING;  /*   */
	USARTC0.DATA = irda_sndbf[ns++];
}

ISR(USARTC0_TXC_vect)
{
	BYTE tmp;


	if(irda_rcvfl != IRDA_STATUS_TRANSMITTING)	return;

	if (ns < 4)
		USARTC0.DATA = irda_sndbf[ns++];
	else
		{
		TCC0.CTRLB 		= TC_WGMODE_NORMAL_gc;
		TCC0.CTRLA 		= TC_CLKSEL_OFF_gc;
		PORTC_OUTCLR=0b00000010;
		
		/*    */
		irda_rcvfl	= IRDA_STATUS_RECEIVING;
		}
}


/*   */
void irda_rcvstart()
{
//	if (irda_rcvfl == IRDA_STATUS_RECEIVING) return; /*       */
//	if (irda_rcvfl == IRDA_STATUS_TRANSMITTING) return; /*       */
//
//	for(nr=0; nr < sizeof(IRDA_RCVRC); nr++) irda_rcvbf[nr]=0;
//	nr=0;
//
//	/*    */
//	irda_rcvfl	= IRDA_STATUS_RECEIVING;
}

/*   ,          ,    */
ISR(USARTC0_RXC_vect)
{
	BYTE tmp;

	if(irda_rcvfl != IRDA_STATUS_RECEIVING) 
		{
		tmp = USARTC0.DATA;
		return;
		}

	irda_rcvbf[nr++]= USARTC0.DATA;
	if (nr == sizeof(IRDA_RCVRC))
		{
		if ((irda_rcvrec.sign == SIGN) && (irda_rcvrec.crc == Crc8(irda_rcvbf,5)))
			irda_rcvfl = IRDA_STATUS_RCVSUCCESS;
		else
			irda_rcvfl = IRDA_STATUS_DISABLE;
		}
}



BYTE irda_rcvstatus(BYTE* _key,BYTE* _rs,BYTE* _pos)
{
	*_key	=irda_rcvrec.key;
	*_rs	=irda_rcvrec.rs;
	*_pos	=irda_rcvrec.ps;
//	*_pos	=irda_rcvrec.crc;

//	return irda_rcvrec.sign;
//	return irda_rcvfl;
	return (irda_rcvfl == IRDA_STATUS_RCVSUCCESS)? 1: 0;
}

void irda_rcvitem(BYTE *_type,BYTE *nm,DWORD *_size)
{
	BYTE i;
	
	*_type	=irda_rcvrec.tp;
	*_size	=irda_rcvrec.size;

	for(i=0; i < 40; i++) nm[i]=irda_rcvrec.name[i];
}

