/*----------------------------------------------------------------------*/
/*      LCD output							 */
/*-----------------------------------------------------------------------*/

#include <avr/io.h>
#include <avr/pgmspace.h>
// #include "symvol.h"
#include "lcd_io.h"

void delay (BYTE dly);

static void lcd_wr ( BYTE cd, UINT dt)
{

	if(cd) 
		PORTD_OUTSET=0b00100000; 
	else	
		PORTD_OUTCLR=0b00100000; 

	PORTD_OUTCLR	=0b01000000; /* WR = L */ 

	PORTE_DIR		=0b11111111; 
	PORTE_OUT		=(BYTE)( dt & 0x00FF); 

	PORTF_DIR		=0b11111111; 
	PORTF_OUT		=(BYTE)( (dt & 0xFF00) >> 8); 

	PORTD_OUTSET	=0b01000000; /* WR = H */ 

	PORTE_DIR		=0b00000000; 
	PORTF_DIR		=0b00000000; 
}

static UINT lcd_rd ()
{
	UINT hi,lo;
	UINT dt;	

	PORTD_OUTSET=0b00100000; 


	PORTD_OUTCLR=0b10000000; /* RD = L */ 

	PORTE_DIR		=0b00000000; 
	PORTF_DIR		=0b00000000; 

	lo=PORTE_IN; 
	hi=PORTF_IN;
	
	dt= ((hi << 8) & 0xFF00) | (lo & 0x00FF); 

	PORTD_OUTSET=0b01000100; /* WR = H */ 

	return dt;

}

static void lcd_cmdwr ( UINT cm, UINT dt)
{
	lcd_wr (0, cm);
	lcd_wr (1, dt);
}

static UINT lcd_cmdrd ( UINT cm)
{
	lcd_wr (0, cm);
	return lcd_rd ();
}


static UINT sqrt(UINT L)
{
    UINT temp, div; 
	UINT rslt = L; 

	if (L == 0)
		return 0; 
	else 
		div = L; 

	while (1)
		{
		temp = L/div + div; 
		div = temp >> 1; 
		div += temp & 1; 
		if (rslt > div) 
			rslt = div; 
		else 
			{
		    if (L/rslt == rslt-1 && L%rslt==0) 
				rslt--; 
		    return rslt; 
		    }
		}
}



/*-----------------------------------------------------------------------*/
/*                                                 */
/*-----------------------------------------------------------------------*/

extern unsigned char Arial24[] PROGMEM;

static BYTE lcd_bitmap ( BYTE chr, BYTE nbt, BYTE pck, UINT clr, UINT fon)
{
	BYTE i, j, *bp, nb, bt, msk;
	BYTE cbf[54];
	UINT bgr;

	memcpy_P(cbf,Arial24+chr*54,54);


	if (pck)
		{
		for ( i=0; (i < 54) && (cbf[53-i] == 0); i++);
		nbt-= (BYTE) (i/3);
		}

	nb=nbt*3;
	for(bp=cbf,i=0; i < nb; i++)
		{
		bt= *bp++;
		for(msk=0b00000001, j=0; j < 8; j++, msk <<=1)
			if( bt & msk)
				lcd_wr (1, clr);
			else
				lcd_wr (1, fon);
		}
	return nbt;
}


static BYTE lcd_char ( BYTE chr, BYTE pck, UINT clr, UINT fon)
{
	if		(chr == 0x20) 					 return lcd_bitmap(0,3,0,clr,fon);
	else if	((0x21 <= chr) && (chr <= 0x7F)) return lcd_bitmap(chr-0x20,18,pck,clr,fon); //   ASCII

	else if	((0x80 <= chr) && (chr <= 0xAF)) return lcd_bitmap(chr-0x20,18,pck,clr,fon); //  

	else if	((0xE0 <= chr) && (chr <= 0xFF)) return lcd_bitmap(chr-0x50,18,pck,clr,fon);
	else 									 return 0;
}

/*-----------------------------------------------------------------------*/
/*                                      */
/*-----------------------------------------------------------------------*/
static void lcd_goto ( UINT r, UINT c, UINT wd)
{
	lcd_cmdwr(0x0020, r); 		// GRAM horizontal Address
	lcd_cmdwr(0x0021, c); 		// GRAM Vertical Address

	lcd_cmdwr(0x0050, r); 		// Horizontal GRAM Start Address
	lcd_cmdwr(0x0051, r+wd-1);  // Horizontal GRAM End Address
	lcd_cmdwr(0x0052, c); 		// Vertical GRAM Start Address
	lcd_cmdwr(0x0053, 0x013F); 	// Vertical GRAM End Address

	lcd_wr(0,0x0022); 			// Write data to DRAM

	return;
}





/*--------------------------------------------------------------------------

   Public Functions

---------------------------------------------------------------------------*/

UINT lcd_color(BYTE r, BYTE g, BYTE b)
{

	return ((UINT)(r) << 11) | ((UINT)(g) << 5) | b; 
}

UINT lcd_colormns(UINT clr)
{
	BYTE r, g, b;
	
	r = (BYTE) ((clr & 0b1111100000000000) >> 11);
	g = (BYTE) ((clr & 0b0000011111100000) >> 5);
	b = (BYTE) ((clr & 0b0000000000011111));
	if	(r) r--;
	if	(g > 1) g-=2; else if	(g) g--;
	if	(b) b--;


	return lcd_color(r, g, b); 
}

UINT lcd_colorpls(UINT clr)
{
	BYTE r, g, b;
	
	r = (BYTE) ((clr & 0b1111100000000000) >> 11);
	g = (BYTE) ((clr & 0b0000011111100000) >> 5);
	b = (BYTE) ((clr & 0b0000000000011111));
	if(r < 31) r++;
	if(g < 62) g+=2; else if(g < 63) g++;
	if(b < 31) b++;

	return lcd_color(r, g, b); 
}





/*-----------------------------------------------------------------------*/
/* Initialize LCD                                                        */
/*-----------------------------------------------------------------------*/
void lcd_initialize (void)
{
	UINT i,j;


	/* SELECT */
	PORTD_OUTCLR=0b00010000; /* CS=L  */ 
	PORTD_OUTSET=0b11000000; /* WR0,WR1=H */ 
	delay(200);

	/* RESET */
	PORTA_OUTCLR=0b00000010; /* RESET=L  */ 
	delay(20);
	PORTA_OUTSET=0b00000010; /* RESET=H  */ 
	delay(120);


	lcd_cmdwr(0x0001, 0x0100); // set SS and SM bit 0100
	lcd_cmdwr(0x0002, 0x0700); // set 1 line inversion
	lcd_cmdwr(0x0003, 0x1030); // set GRAM write direction and BGR=1.
	lcd_cmdwr(0x0004, 0x0000); // Resize register
	lcd_cmdwr(0x0008, 0x0302); // set the back porch and front porch
	lcd_cmdwr(0x0009, 0x0000); // set non-display area refresh cycle ISC[3:0]
	lcd_cmdwr(0x000A, 0x0000); // FMARK function
	lcd_cmdwr(0x000C, 0x0000); // RGB interface setting
	lcd_cmdwr(0x000D, 0x0000); // Frame marker Position
	lcd_cmdwr(0x000F, 0x0000); // RGB interface polarity
	delay(120);

	//*************Power On sequence ****************//
	lcd_cmdwr(0x0010, 0x0790); // SAP=0, BT[3:0]=111, AP=1001, DSTB=0, SLP=0, STB=0
	lcd_cmdwr(0x0011, 0x0005); // DC1[2:0]=000, DC0[2:0]=000, VC[2:0]=101
	lcd_cmdwr(0x0012, 0x0000); // VCIRE=0 VREG1OUT voltage
	lcd_cmdwr(0x0013, 0x0000); // VDV[4:0]=00000 for VCOM amplitude
	delay(120);

	lcd_cmdwr(0x0010, 0x15B0); // SAP=1, BT[3:0]=101, AP=1011, DSTB=0, SLP=0, STB=0 
	delay(50);
	lcd_cmdwr(0x0011, 0x0005); // DC1[2:0]=000, DC0[2:0]=000, VC[2:0]=011 (vci=vdd)
	delay(50); // Delay 50ms
	lcd_cmdwr(0x0012, 0x008c); // Internal reference voltage= Vci (GVDD = 4,5V), VCIRE=1, VRH[3:0]=1100; 
	delay(50); // Delay 50ms
	lcd_cmdwr(0x0013, 0x1700); // Set VDV[4:0]=10111 for VCOM=GVDD*1,16=5,22V amplitude 
	lcd_cmdwr(0x0029, 0x0020); // Set VCM[5:0]=100000 for VCOMH=GVDD*0,845=3,8V
	lcd_cmdwr(0x002B, 0x000c); // Set Frame Rate FRS[3:0]=1100
	delay(50); // Delay 50ms

	// ----------- Adjust the Gamma Curve ----------//
	lcd_cmdwr(0x0030, 0x0000);
	lcd_cmdwr(0x0031, 0x0106);
	lcd_cmdwr(0x0032, 0x0101);
	lcd_cmdwr(0x0035, 0x0106);
	lcd_cmdwr(0x0036, 0x0203);
	lcd_cmdwr(0x0037, 0x0000);
	lcd_cmdwr(0x0038, 0x0707);
	lcd_cmdwr(0x0039, 0x0204);
	lcd_cmdwr(0x003C, 0x0106);
	lcd_cmdwr(0x003D, 0x0103);
	//------------------ Set GRAM area ---------------//
	lcd_cmdwr(0x0020, 0x0000); // GRAM horizontal Address
	lcd_cmdwr(0x0021, 0x0000); // GRAM Vertical Address
	lcd_cmdwr(0x0050, 0x0000); // Horizontal GRAM Start Address
	lcd_cmdwr(0x0051, 0x00EF); // Horizontal GRAM End Address
	lcd_cmdwr(0x0052, 0x0000); // Vertical GRAM Start Address
	lcd_cmdwr(0x0053, 0x013F); // Vertical GRAM Start Address

	lcd_cmdwr(0x0060, 0x2700); // Gate Scan Line 2700
	lcd_cmdwr(0x0061, 0x0001); // NDL,VLE, REV
	lcd_cmdwr(0x006A, 0x0000); // set scrolling line
	lcd_cmdwr(0x0090, 0x0033);
	lcd_cmdwr(0x0095, 0x0110);
	//-------------- Partial Display Control ---------//
//	lcd_cmdwr(0x0080, 0x0000);
//	lcd_cmdwr(0x0081, 0x0000);
//	lcd_cmdwr(0x0082, 0x0000);
//	lcd_cmdwr(0x0083, 0x0000);
//	lcd_cmdwr(0x0084, 0x0000);
//	lcd_cmdwr(0x0085, 0x0000);
	//-------------- Panel Control -------------------//
//	lcd_cmdwr(0x00FF, 0x0001);
//	lcd_cmdwr(0x00FF, 0x000C);
//	lcd_cmdwr(0x00FF, 0x0000);
	delay(100);
	lcd_cmdwr(0x0007, 0x0133); // 262K color and display ON PTDE[1:0]=00, BASEE=1, GON=1, DTE=1, CL=0, D[1:0]=11
	delay(50);


	lcd_wr(0,0x0022); // Write data to DRAM
	for(j=0; j < 320; j++)
	for(i=0; i < 240; i++)
		lcd_wr (1, 0x0000);
}

void lcd_enter_pwrsave (void)
{
	lcd_cmdwr(0x0007, 0x0131); // 262K color and display ON PTDE[1:0]=00, BASEE=1, GON=1, DTE=1, CL=0, D[1:0]=01
	delay(50);
	lcd_cmdwr(0x0007, 0x0020); // 262K color and display ON PTDE[1:0]=00, BASEE=0, GON=1, DTE=0, CL=0, D[1:0]=00
	delay(50);
	//Into Standby Mode
	lcd_cmdwr(0x0010, 0x0002); // SAP=0, BT[3:0]=000, AP=0000, DSTB=0, SLP=1, STB=0 

}
void lcd_exit_pwrsave (void)
{
	lcd_cmdwr(0x0010, 0x15B0); // SAP=1, BT[3:0]=101, AP=1011, DSTB=0, SLP=0, STB=0 
	delay(50);
	lcd_cmdwr(0x0007, 0x0133); // 262K color and display ON PTDE[1:0]=00, BASEE=1, GON=1, DTE=1, CL=0, D[1:0]=11
	delay(50);
}



void lcd_ledon()
{
	PORTD_OUTSET 		= 0b00001010; 

}


void lcd_ledoff()
{
	PORTD_OUTCLR 		= 0b00001111; 

}


void lcd_square(UINT r1, UINT c1, UINT r2, UINT c2, UINT clr, UINT dlt, UINT dcn)
{
	UINT i,j,n,m;
	UINT arclr[240];
	
	

	n=(r2-r1+1);
	m=(c2-c1+1);
	
	for(i=0 ; i < n; i++) arclr[i]=clr;
	for(i=dlt ; i ; i--) arclr[i-1]=lcd_colorpls(arclr[i]);
	for(i=n-dcn ; i < n ; i++) 
		arclr[i]=lcd_colormns(arclr[i-1]);


	lcd_goto(r1, c1, n);
	for(j=0; j < m; j++) 
		for(i=0 ; i < n; i++)
			lcd_wr(1, arclr[i]);
}

void lcd_circle(UINT y, UINT x, UINT r, UINT clr, UINT fon)
{

	UINT i,j,a;
	UINT clrpls[5], clrmns[5];

	clrpls[0]=lcd_colorpls(clr);	
	clrmns[0]=lcd_colormns(clr);	
	for(i = 1; i < 5; i++)
		{
		clrpls[i]=lcd_colorpls(clrpls[i-1]);	
		clrmns[i]=lcd_colormns(clrmns[i-1]);	
		}

	lcd_goto(y-r, x-r,2*r+1);

	for(j=r; j > 0;j--)
		{
		a=sqrt(r*r-j*j);
		for(i=r; i > 0; i--)
			if(i < a)
				if (a-i <= 4)
					if( 5-(a-i) < (i-1))
						lcd_wr (1, clrpls[5-(a-i)]);
					else
						lcd_wr (1, clrpls[i-1]);
				else
					lcd_wr (1, clr);
			else
				lcd_wr (1, fon);
		for(i=0; i <= r; i++)
			if(i < a) 	
				if (a-i <= 4)
					if( 5-(a-i) < (i-1))
						lcd_wr (1, clrmns[5-(a-i)]);
					else
						lcd_wr (1, clrmns[i-1]);
				else
					lcd_wr (1, clr);
			else
				lcd_wr (1, fon);
		}

	for(j=0; j <= r; j++)
		{
		a=sqrt(r*r-j*j);
		for(i=r; i > 0; i--)
			if(i < a) 	
				if (a-i <= 4)
					if( 5-(a-i) < (i-1))
						lcd_wr (1, clrpls[5-(a-i)]);
					else
						lcd_wr (1, clrpls[i-1]);
				else
					lcd_wr (1, clr);
			else
				lcd_wr (1, fon);
		for(i=0; i <= r; i++)
			if(i < a) 	
				if (a-i <= 4)
					if( 5-(a-i) < (i-1))
						lcd_wr (1, clrmns[5-(a-i)]);
					else
						lcd_wr (1, clrmns[i-1]);
				else
					lcd_wr (1, clr);
			else
				lcd_wr (1, fon);
		}
}	

BYTE lcd_output(UINT r, UINT c, UINT n, BYTE *pnt, UINT clr, UINT fon)
{
	UINT i;
	BYTE j;
	BYTE spc= 0;	

	lcd_goto(r,c,24);
	for(i=0,j=0; pnt[j]; j++)
		{
		i+=lcd_char(pnt[j], 1, clr, fon);

		if ( (n - i) < 18+3 ) break;
		i+=lcd_bitmap(spc,3,0, clr, fon);	
		}

	for(;i < n;i++)
		lcd_bitmap(spc,1,0, clr, fon);	

	return j+1;
}

void lcd_outchar(UINT r, UINT c, UINT n, BYTE chr, UINT clr, UINT fon)
{
	UINT i;
	BYTE spc= 0;	

	lcd_goto(r,c,24);
	i=lcd_char(chr, 1, clr, fon);

	for(;i < n;i++)
		lcd_bitmap(spc,1,0, clr, fon);	

}


void lcd_outbitmap(BYTE r, BYTE c, BYTE n, BYTE *pnt, BYTE inv)
{
	lcd_goto(r,c,24);
	lcd_bitmap(pnt,n,0,0xFFFF, 0x0000);	
}




