#include <avr/io.h>
#include <avr/interrupt.h>

#include "integer.h"
#include "irda_io.h"
#include "button.h"
#include "scrn_err.h"
#include "scrn_sel.h"
#include "scrn_play.h"

volatile static BYTE State;
typedef enum ST_enum {
	ST_Stop				= 0,
	ST_Start			= 1, 
	ST_NoLink1			= 2,
	ST_Busy				= 3,
	ST_GetKey			= 4
	} ST_type;

typedef enum SCRN_enum {
	SCRN_POWERSAVE		= 0,
	SCRN_ERROR			= 1, 
	SCRN_SELECT			= 2,
	SCRN_PLAY			= 3
	} SCRN_type;


#define BTCLR 0b0101010101010111
#define RD    0b1111100000000000
#define GR    0b0000011111100000


static BUTTON btuttons[4] = {
	{ bsl_emp,  0,  0,  0,  0,  0,     0, ""},
	{ bsl_pwr, 50,  0,200,319, 56,  0, 0,     0, ""},
	{ bsl_lnk, 10,298, 41,319, 14,300, 0, BTCLR, {247}}
	};


 
static BYTE scrn_State();
void pwr_save();

void scrn()
{
	BYTE rc;

	for(;;)
		{
		bt_lstclr();
		bt_lstadd(btuttons+1);
		bt_lstadd(btuttons+2);
		bt_lstviwe();
		State=ST_Start;

		while (State) rc=scrn_State();
		bt_lstclr();
		lcd_square(0, 0 , 239, 319, 0, 0, 0);

		switch (rc)
			{
			case SCRN_POWERSAVE:
				pwr_save();
				break;
			case SCRN_ERROR:
				if (scrn_err()) pwr_save();
				break;
			case SCRN_SELECT:
				if (scrn_sel()) pwr_save();
				break;
			case SCRN_PLAY:
				if (scrn_play()) pwr_save();
				break;
			}
		}


}

extern volatile UINT TimerSt;

static BYTE scrn_State()
{
	BYTE rc;
	BYTE _key, _rs, _pos;

	if( bt_press() == bsl_pwr)
		{
		State=ST_Stop;
		while(bt_press());
		return SCRN_POWERSAVE;
		}

	if(TimerSt) return;

	switch (State)
		{
		case ST_Start:
			bt_OnLink();
			irda_send(CM_GetStatus,0);
			State=ST_GetKey;
			TimerSt=200;
			break;

		case ST_GetKey:
			rc=irda_rcvstatus(&_key,&_rs,&_pos);
			if (!rc)
				State=ST_NoLink1;
			else
				{
				bt_OnLink();
				if (!_rs)
					State=ST_Busy;
				else if (_rs & MODE_ERROR)
					{
					State=ST_Stop;
					return SCRN_ERROR;
					}
				else if (_rs & MODE_SELECT)
					{
					State=ST_Stop;
					return SCRN_SELECT;
					}
				else if (_rs & MODE_PLAY)
					{
					State=ST_Stop;
					return SCRN_PLAY;
					}
				else
					{
					State=ST_NoLink1;
					TimerSt=200;
					}
				}
			break;

		case ST_NoLink1:
			bt_NoLink();
		case ST_Busy:
			irda_send(CM_GetStatus,0);
			State=ST_GetKey;
			TimerSt=500;
			break;

		}

	if( !TimerSt ) TimerSt=5;
	return 0;
}

