#include <avr/io.h>
#include <avr/interrupt.h>

#include "integer.h"
#include "lcd_io.h"
#include "irda_io.h"
#include "button.h"
#include "scrn_err.h"

volatile static BYTE State;

typedef enum ST_enum {
	ST_Stop				= 0,
	ST_Start			= 1, 
	ST_NoLink1			= 2,
	ST_Busy				= 3,
	ST_GetKey			= 4,
	ST_NoLink2			= 5,
	ST_GetName	 		= 6
	} ST_type;


#define BTCLR 0b0101010101010111
#define RD    0b1111100000000000
#define GR    0b0000011111100000


static BUTTON btuttons[10] = {
	{ bsl_emp,  0,  0,  0,  0,  0,     0, ""},
	{ bsl_pwr, 70, 70,170,250, 70, 70, 0,     0, ""},
	{ bsl_lnk, 10,298, 41,319, 14,300, 0, BTCLR, {247}}
	};


 
static BYTE scrn_errState();

BYTE scrn_err()
{
	BYTE rc;

	bt_lstclr();
	bt_lstadd(btuttons+1);
	bt_lstadd(btuttons+2);
	bt_lstviwe();
	State=ST_Start;

	while (State) rc=scrn_errState();

	bt_lstclr();
	lcd_square(0, 0 , 239, 319, 0, 0, 0);
	return rc;
}


extern volatile UINT TimerSt;

static BYTE scrn_errState()
{
	BYTE rc;
	static BYTE cur_key;
	BYTE _key, _rs, _pos;
	BYTE tp, nm[65];
	DWORD sz;

	if( bt_press() == bsl_pwr)
		{
		State=ST_Stop;
		while(bt_press());
		return 1;
		}

	if(TimerSt) return 0;

	switch (State)
		{
		case ST_Start:
			bt_OnLink();
			irda_send(CM_GetStatus,0);
			State=ST_GetKey;
			TimerSt=200;
			break;

		case ST_GetKey:
			rc=irda_rcvstatus(&_key,&_rs,&_pos);
			if (!rc)
				State=ST_NoLink1;
			else
				{
				bt_OnLink();
				if (!_rs)
					State=ST_Busy;
				else if (!(_rs & MODE_ERROR))
					State=ST_Stop;
				else
					{
					cur_key=_key;
					irda_send(CM_GetElement,0);
					State=ST_GetName;
					TimerSt=200;
					}
				}
			break;

		case ST_NoLink1:
			bt_NoLink();

		case ST_Busy:
			irda_send(CM_GetStatus,0);
			State=ST_GetKey;
			TimerSt=200;
			break;

		case ST_GetName:
			rc=irda_rcvstatus(&_key,&_rs,&_pos);
			if (!rc)
				State=ST_NoLink2;
			else if (cur_key != _key)
				State=ST_Start;
			else
				{
				bt_OnLink();
				if(_rs)
					{
					irda_rcvitem(&tp,nm,&sz);
					lcd_output( 84, 10,309, nm, lcd_color(31,63,31), 0x0000);

					irda_send(CM_GetStatus,0);
					TimerSt=1000;
					}
				State=ST_GetName;
				}
			break;

		case ST_NoLink2:
			bt_NoLink();
			irda_send(CM_GetStatus,0);
			State=ST_GetName;
			TimerSt=200;
			break;
		}

	if( !TimerSt ) TimerSt=5;

	return 0;
}
