#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>

#include "integer.h"
#include "lcd_io.h"
#include "irda_io.h"
#include "button.h"
#include "scrn_play.h"

volatile static BYTE State;

typedef enum ST_enum {
	ST_Stop				= 0,
	ST_Start			= 1, 
	ST_NoLink1			= 2,
	ST_Busy				= 3,
	ST_GetKey			= 4,
	ST_NoLink2			= 5,
	ST_GetName	 		= 6,
	ST_GetButton	 	= 7,
	ST_Refresh	 		= 8,
	ST_Escape			= 9,
	ST_Play				=10,
	ST_Prev				=11,
	ST_Next				=12,
	ST_NoLink3			=13
	} ST_type;


typedef enum PCM_FMT_enum
{
    PCM_FAILE	= 0,  
    PCM_16_44	= 1,  
    PCM_16_88	= 2,  
    PCM_16_176	= 3,  
    PCM_16_352	= 4,  
    PCM_24_44	= 5,  
    PCM_24_88	= 6,  
    PCM_24_176	= 7,  
    PCM_24_352	= 8,  
    PCM_16_48	= 9,  
    PCM_16_96	= 10,  
    PCM_16_192	= 11,  
    PCM_16_384	= 12,  
    PCM_24_48	= 13,  
    PCM_24_96	= 14,  
    PCM_24_192	= 15,  
    PCM_24_384	= 16  
} PCM_FMT_t;

static char PCM_FMT_ttl[18][20]= {
    "",  
    "[2x16/44]",  
    "[2x16/88]",  
    "[2x16/176]",  
    "[2x16/352]",  
    "[2x24/44]",  
    "[2x24/88]",  
    "[2x24/176]",  
    "[2x24/352]",  
    "[2x16/48]",  
    "[2x16/96]",  
    "[2x16/192]",  
    "[2x16/384]",  
    "[2x24/48]",  
    "[2x24/96]",  
    "[2x24/192]",
    "[2x24/384]"
	};  


#define BTCLR 0b0101010101010111
#define RD    0b1111100000000000
#define GR    0b0000011111100000


static BUTTON btuttons[10] = {
	{ bsl_emp,  0,  0,  0,  0,  0,     0, ""},
	{ bsl_pwr, 80, 70,160,250, 80, 70, 0,     0, ""},
	{ bsl_prv,200,  0,231, 78,204, 28, 0, BTCLR, {246, 246}},
	{ bsl_ply,200, 80,231,158,204,108, 0, BTCLR, {244}},
	{ bsl_stp,200,160,231,238,204,190, 0, BTCLR, {248}},
	{ bsl_nxt,200,240,231,319,204,270, 0, BTCLR, {245, 245}},
	{ bsl_ttl, 10,  0, 41,296, 14, 10, 0, BTCLR, ""},
	{ bsl_lnk, 10,298, 41,319, 14,300, 0, BTCLR, {247}}
	};


 
static BYTE scrn_playState();

BYTE scrn_play()
{
	BYTE rc;

	bt_lstclr();
	bt_lstadd(btuttons+1);
	bt_lstadd(btuttons+2);
	bt_lstadd(btuttons+3);
	bt_lstadd(btuttons+4);
	bt_lstadd(btuttons+5); 
 	bt_lstadd(btuttons+6);
	bt_lstadd(btuttons+7);
	bt_lstviwe();
	State=ST_Start;

	while (State) rc=scrn_playState();

	bt_lstttl(bsl_ttl, "");
	bt_lstclr();
	lcd_square(0, 0 , 239, 319, 0, 0, 0);
	return rc;
}

void scr_viwetrc(UINT pos);
void scr_nametrc();
void scr_refrtrc(BYTE rs, UINT pos);

extern volatile UINT TimerSt;

static BYTE scrn_playState()
{
	BYTE rc;
	static BYTE cur_key;
	BYTE _key, _rs, _pos;
	BYTE tp, nm[65];
	DWORD sz;
	static UINT refrcnt;

	if( bt_press() == bsl_pwr)
		{
		State=ST_Stop;
		while(bt_press());
		return 1;
		}

	if(TimerSt) return 0;

	switch (State)
		{
		case ST_Start:
			bt_OnLink();
			irda_send(CM_GetStatus,0);
			State=ST_GetKey;
			TimerSt=200;
			break;

		case ST_GetKey:
			rc=irda_rcvstatus(&_key,&_rs,&_pos);
			if (!rc)
				State=ST_NoLink1;
			else
				{
				bt_OnLink();
				if (!_rs)
					State=ST_Busy;
				else if (!(_rs & MODE_PLAY))
					State=ST_Stop;
				else
					{
					cur_key=_key;
					scr_viwetrc(_pos);
					irda_send(CM_GetElement,0);
					State=ST_GetName;
					TimerSt=200;
					}
				}
			break;

		case ST_NoLink1:
			bt_NoLink();

		case ST_Busy:
			irda_send(CM_GetStatus,0);
			State=ST_GetKey;
			TimerSt=200;
			break;

		case ST_GetName:
			rc=irda_rcvstatus(&_key,&_rs,&_pos);
			if (!rc)
				State=ST_NoLink2;
			else if (cur_key != _key)
				State=ST_Start;
			else
				{
				bt_OnLink();
				if(_rs)
					{
					scr_nametrc();
					refrcnt=0;
					}
				State=ST_GetButton;
				}
			break;

		case ST_NoLink2:
			bt_NoLink();
			irda_send(CM_GetElement,0);
			State=ST_GetName;
			TimerSt=200;
			break;


		case ST_Refresh:
			rc=irda_rcvstatus(&_key,&_rs,&_pos);
			if (!rc)
				State=ST_NoLink3;
			else if (cur_key != _key)
				State=ST_Start;
			else
				{
				bt_OnLink();
				if(_rs)
					{
					scr_refrtrc(_rs, _pos);
					refrcnt=0;
					}
				State=ST_GetButton;
				}
			break;

		case ST_NoLink3:
			bt_NoLink();
			irda_send(CM_GetElement,0);
			State=ST_Refresh;
			TimerSt=200;
			break;


		case ST_GetButton:
			bt_OnLink();
			State=ST_GetButton;
			switch(bt_press())
				{
				case bsl_ttl:
					State=ST_Escape;
					irda_send(CM_Escape,0);
					TimerSt=200;
					break;
				case bsl_ply:
					State=ST_Refresh;
					irda_send(CM_Play,0);
					TimerSt=200;
					break;
				case bsl_stp:
					State=ST_Stop;
					irda_send(CM_Stop,0);
					TimerSt=200;
					break;
				case bsl_prv:
					State=ST_Prev;
					irda_send(CM_Prev,0);
					TimerSt=200;
					break;
				case bsl_nxt:
					State=ST_Next;
					irda_send(CM_Next,0);
					TimerSt=200;
					break;
				default:
					if(refrcnt++ > 70)
						{
						irda_send(CM_GetElement,0);
						State=ST_Refresh;
						TimerSt=200;
						}
					break;
				}
			break;

		case ST_Escape:
		case ST_Stop:
			rc=irda_rcvstatus(&_key,&_rs,&_pos);
			if (!rc)
				State=ST_NoLink3;
			else if (cur_key != _key)
				State=ST_Start;
			else
				State=ST_Stop;
			break;

		case ST_Play:
			rc=irda_rcvstatus(&_key,&_rs,&_pos);
			if (!rc)
				State=ST_NoLink3;
			else if (cur_key != _key)
				State=ST_Start;
			else
				{
				while(bt_press());
				State=ST_Refresh;
				}
			break;	

		case ST_Prev:
		case ST_Next:
			rc=irda_rcvstatus(&_key,&_rs,&_pos);
			if (!rc)
				State=ST_NoLink3;
			else if (cur_key != _key)
				State=ST_Start;
			else
				State=ST_Start;
			break;	

		}

	if( !TimerSt ) TimerSt=5;

	return 0;
}



void scr_viwetrc(UINT pos)
{
	BYTE tp, nm[65];
	DWORD sz;
	BYTE tm[10];

	irda_rcvitem(&tp,nm,&sz);
	bt_lstttl(bsl_ttl, nm);

	lcd_outchar(52, 10, 18, 242, lcd_color( 0,0,31), 0x0000);
	
	strcpy(nm, "[");
	strint(tm, pos+1);
	strcat(nm,tm);
	strcat(nm,"/");
	strint(tm, (int)sz);
	strcat(nm,tm);
	strcat(nm,"]");

	lcd_output( 52, 38,200, nm, lcd_color( 0,0,31), 0x0000);

	/*   */
	lcd_square(174, 6, 184, 306, lcd_color( 29,60,29), 4, 4);


}

void scr_nametrc()
{

	BYTE tp, nm[65], i;
	DWORD sz;
	BYTE tm[10];
	DWORD secnd; 
	BYTE scnd,mint;

	irda_rcvitem(&tp,nm,&sz);
	
/*   */
	i =	lcd_output( 84, 10,309,   nm, lcd_color(31,63,31), 0x0000);
	if ( i < strlen(nm) )
		lcd_output(112, 10,309, nm+i, lcd_color(31,63,31), 0x0000);
	else
		lcd_output(112, 10,309,   "", lcd_color(31,63,31), 0x0000);

/*   */
	if (tp== PCM_16_44)
		secnd=sz/2/2/44100;
	else if (tp == PCM_16_88)
		secnd=sz/2/2/88200;
	else if (tp == PCM_16_176)
		secnd=sz/2/2/176400;

	else if (tp == PCM_24_44)
		secnd=sz/2/3/44100;
	else if (tp == PCM_24_88)
		secnd=sz/2/3/88200;
	else if (tp == PCM_24_176)
		secnd=sz/2/3/176400;

	else if (tp == PCM_16_48)
		secnd=sz/2/2/48000;
	else if (tp == PCM_16_96)
		secnd=sz/2/2/96000;
	else if (tp == PCM_16_192)
		secnd=sz/2/2/192000;

	else if (tp == PCM_24_48)
		secnd=sz/2/3/48000;
	else if (tp == PCM_24_96)
		secnd=sz/2/3/96000;
	else if (tp == PCM_24_192)
		secnd=sz/2/3/192000;

	lcd_output(144, 10,180, PCM_FMT_ttl[tp], lcd_color( 0,0,31), 0x0000);

/*   */
	mint=secnd/60;
	scnd=secnd%60;

	tm[1]='0'+(mint/10);
	tm[2]='0'+(mint%10);
	tm[3]=':';
	tm[4]='0'+(scnd/10);
	tm[5]='0'+(scnd%10);

	tm[0]='[';
	tm[6]=']';
	tm[7]='\0';

	lcd_output(144, 230,88, tm, lcd_color( 0,0,31), 0x0000);
}


static char pbttnm[2][4]= {{244,0},
					  {249,0}};

void scr_refrtrc(BYTE rs, UINT pos)
{

	BYTE tp, nm[65];
	DWORD sz;

	irda_rcvitem(&tp,nm,&sz);
	
/*      */
	if( rs & MODE_PAUSE )
		bt_lstttl(bsl_ply, pbttnm[0]);
	else
		bt_lstttl(bsl_ply, pbttnm[1]);

/*   */
	lcd_square(174, 6 , 184, 6+pos*3, lcd_color( 0,45,0), 4, 4);

}

