#include <avr/io.h>
#include <avr/interrupt.h>

#include "integer.h"
#include "lcd_io.h"
#include "irda_io.h"
#include "button.h"
#include "plist.h"
#include "scrn_sel.h"

volatile static BYTE State;
typedef enum ST_enum {
	ST_Stop				= 0,
	ST_Start			= 1, 
	ST_NoLink1			= 2,
	ST_Busy				= 3,
	ST_GetKey			= 4,
	ST_ReloadScrn		= 5,
	ST_NoLink2			= 6,
	ST_GetButton	 	= 7,
	ST_Escape			= 8,
	ST_Play				= 9,
	ST_Prev				=10,
	ST_Next				=11,
	ST_TstLink			=12,
	ST_NoLink3			=13
	} ST_type;


#define BTCLR 0b0101010101010111
#define RD    0b1111100000000000
#define GR    0b0000011111100000


static BUTTON btuttons[10] = {
	{ bsl_emp,  0,  0,  0,  0,  0,     0, ""},
	{ bsl_pwr, 80, 70,160,250, 80, 70, 0,     0, ""},
	{ bsl_prv,200,  0,231,118,204, 48, 0, BTCLR, {246, 246}},
	{ bsl_ply,200,120,231,198,204,158, 0, BTCLR, {244}},
	{ bsl_stp,200,160,231,238,204,200, 0, BTCLR, ""},
	{ bsl_nxt,200,200,231,319,204,250, 0, BTCLR, {245, 245}},
	{ bsl_ttl, 10,  0, 41,296, 14, 10, 0, BTCLR, "	"},
	{ bsl_lnk, 10,298, 41,319, 14,300, 0, BTCLR, {247}}
	};

 
BYTE scrn_selState();

BYTE scrn_sel()
{
	BYTE rc;

	bt_lstclr();
	bt_lstadd(btuttons+1);
	bt_lstadd(btuttons+2);
	bt_lstadd(btuttons+3);
	bt_lstadd(btuttons+5); 
 	bt_lstadd(btuttons+6);
	bt_lstadd(btuttons+7);
	bt_lstviwe();
	State=ST_Start;


	while (State) rc=scrn_selState();

	bt_lstttl(bsl_ttl, "");
	bt_lstclr();
	lcd_square(0, 0 , 239, 319, 0, 0, 0);

	return rc;
}


extern volatile UINT TimerSt;
BYTE scrn_selState()
{
	BYTE rc;
	static BYTE cur_key, cur_pos, st_pos, ac_pos, TstLink_cnt;
	BYTE _key, _rs, _pos;
	BYTE tp, nm[65];
	DWORD sz;

	if( bt_press() == bsl_pwr)
		{
		State=ST_Stop;
		while(bt_press());
		return 1;
		}

	if(TimerSt) return 0;

	switch (State)
		{
		case ST_Start:
			st_pos=0;
			cur_pos=0;
			cur_key=0;
			plist_clr();
			bt_OnLink();
			irda_send(CM_GetStatus,0);
			State=ST_GetKey;
			TimerSt=200;

			break;

		case ST_GetKey:
			rc=irda_rcvstatus(&_key,&_rs,&st_pos);
			if (!rc)
				State=ST_NoLink1;
			else
				{
				bt_OnLink();
				if (!_rs)
					State=ST_Busy;
				else if (!(_rs & MODE_SELECT))
					State=ST_Stop;
				else
					{
					plist_clr();
					irda_rcvitem(&tp,nm,&sz);
					bt_lstttl(bsl_ttl, nm);
					cur_key=_key;
					ac_pos=st_pos;
					if( st_pos+MAXROW > sz)
						if ( MAXROW > sz )
							st_pos=0;
						else
							st_pos=sz-MAXROW;
						 
					cur_pos=st_pos;
					plist_strtitem(st_pos);
					irda_send(CM_GetElement,cur_pos);
					State=ST_ReloadScrn;
					TimerSt=200;
					}
				}
			break;

		case ST_NoLink1:
			bt_NoLink();

		case ST_Busy:
			irda_send(CM_GetStatus,0);
			State=ST_GetKey;
			TimerSt=200;
			break;


		case ST_ReloadScrn:
			rc=irda_rcvstatus(&_key,&_rs,&_pos);
			if (!rc)
				State=ST_NoLink2;
			else if (cur_key != _key)
				State=ST_Start;
			else
				{
				bt_OnLink();
				if(_rs)
					{
					cur_pos=_pos;
					plist_viewitem(cur_pos);
					if(cur_pos < MAXROW+st_pos-1 )
						{
						irda_send(CM_GetElement,cur_pos+1);
						State=ST_ReloadScrn;
						TimerSt=200;
						break;
						}
					}
				cur_pos=ac_pos;
				plist_selitem(cur_pos);
				TstLink_cnt=0;
				State=ST_GetButton;
				}
			break;

		case ST_NoLink2:
			bt_NoLink();
			irda_send(CM_GetElement,cur_pos);
			State=ST_ReloadScrn;
			TimerSt=200;
			break;

		case ST_GetButton:
			bt_OnLink();
			State=ST_GetButton;
			switch(bt_press())
				{
				case bsl_ttl:
					State=ST_Escape;
					irda_send(CM_Escape,cur_pos);
					TimerSt=200;
					break;
				case bsl_ply:
					State=ST_Play;
					irda_send(CM_Play,cur_pos);
					TimerSt=200;
					break;
				case bsl_prv:
					if(cur_pos)
						{
						State=ST_Prev;
						irda_send(CM_SetElement,cur_pos-1);
						TimerSt=200;
						}
					break;
				case bsl_nxt:
					State=ST_Next;
					irda_send(CM_SetElement,cur_pos+1);
					TimerSt=200;
					break;
				}

			if(State == ST_GetButton)
				{
				if( TstLink_cnt++ > 200)
					{
					TstLink_cnt=0;
					State = ST_TstLink;
					irda_send(CM_GetStatus,0);
					TimerSt=200;
					}
				}
			else
				TstLink_cnt=0;

			break;

		case ST_TstLink:
			rc=irda_rcvstatus(&_key,&_rs,&_pos);
			if (!rc)
				State=ST_NoLink3;
			else if (cur_key != _key)
				State=ST_Start;
			else
				State=ST_GetButton;
			break;	

		case ST_Escape:
		case ST_Play:
			rc=irda_rcvstatus(&_key,&_rs,&_pos);
			if (!rc)
				State=ST_NoLink3;
			else if (cur_key != _key)
				State=ST_Start;
			else if(_rs)
				State=ST_Start;
			else
				State=ST_GetButton;
			break;	

		case ST_Prev:
		case ST_Next:
			rc=irda_rcvstatus(&_key,&_rs,&_pos);
			if (!rc)
				State=ST_NoLink3;
			else if (cur_key != _key)
				State=ST_Start;
			else
				{ 
				if(_rs)
					{
					cur_pos=_pos;
					plist_selitem(cur_pos);
					}
				State=ST_GetButton;
				}
			break;	

		case ST_NoLink3:
			bt_NoLink();
			irda_send(CM_GetStatus,0);
			State=ST_TstLink;
			TimerSt=200;
			break;
		}

	if( !TimerSt ) TimerSt=5;

	return 0;
}


