#include <avr/io.h>

#include "integer.h"
#include "tchscrn_io.h"

void delay (BYTE dly);




void tchscrn_initialize ()
{

	/* Set up ADC to have signed conversion mode and 12 bit resolution. */
	ADCA.CTRLB = ADC_RESOLUTION_12BIT_gc | ADC_CONMODE_bm; 

	// The ADC has different voltage reference options, controlled by the REFSEL bits in the
	// REFCTRL register. Here the external reference (AREFA - PIN0 PORTA) is selected
	ADCA.REFCTRL = ADC_REFSEL_AREFA_gc;

	// The clock into the ADC decide the maximum sample rate and the conversion time, and
	// this is controlled by the PRESCALER bits in the PRESCALER register. Here, the
	// Peripheral Clock is divided by 128 
	ADCA.PRESCALER = ADC_PRESCALER_DIV128_gc;

	/* Setup channel 0 to have single ended input. */
	ADCA.CH0.CTRL =   ADC_CH_INPUTMODE_SINGLEENDED_gc | ADC_CH_GAIN_1X_gc;
	/* Setup channel 1 to have single ended input. */
	ADCA.CH1.CTRL =   ADC_CH_INPUTMODE_SINGLEENDED_gc | ADC_CH_GAIN_1X_gc;

	// Setting up the which pins to convert.
	// Note that the negative pin is internally connected to ground
	ADCA.CH0.MUXCTRL = ADC_CH_MUXPOS_PIN3_gc | ADC_CH_MUXNEG_PIN2_gc;
	ADCA.CH1.MUXCTRL = ADC_CH_MUXPOS_PIN2_gc | ADC_CH_MUXNEG_PIN3_gc;

	// Before the ADC can be used it must be enabled
	ADCA.CTRLA = ADC_ENABLE_bm;

	// Wait until the ADC is ready
	delay(10);

	return;
}


int tchscrn_reader()
{
	long y;

	PORTA_DIR  		= (PORTA_DIR & 0b11000011) | 0b00010100;
	PORTA_PIN3CTRL	= PORT_OPC_PULLDOWN_gc;	

	PORTA_OUTSET	= 0b00010000;
	PORTA_OUTCLR	= 0b00000100;
	delay(2);
	
	// Start a single conversion
 	ADCA.CH0.CTRL |= ADC_CH_START_bm;

	// Waite conversion
	while( !ADCA.CH0.INTFLAGS);

	/* Clear interrupt flag.*/
	ADCA.CH0.INTFLAGS = ADC_CH_CHIF_bm;

	/* Return result register contents*/
	y = ADCA.CH0.RES;
	y = (y < 320)? -1: (int)((y-320)*240/1460);  
	y = (y > 240)? -1: y;
	return y;
}



int tchscrn_readec()
{
	long x;	

	PORTA_DIR  		= (PORTA_DIR & 0b11000011) | 0b00101000;
	PORTA_PIN2CTRL	= PORT_OPC_PULLDOWN_gc;	

	PORTA_OUTSET	= 0b00100000;
	PORTA_OUTCLR	= 0b00001000;
	delay(2);
	
	// Start a single conversion
 	ADCA.CH1.CTRL |= ADC_CH_START_bm;
	
	// Waite conversion
	while( !ADCA.CH1.INTFLAGS);

	/* Clear interrupt flag.*/
	ADCA.CH1.INTFLAGS = ADC_CH_CHIF_bm;

	/* Return result register contents*/
	x = ADCA.CH1.RES;
	x = (x < 200)? -1: 320-((x-200)*320/1770);
	x = (x <  -1)? -1: x;
	return x;
}


